'''
Created on 2 Dec 2017

@author: julianporter
'''
from setuptools import Command
import shutil
import os

class Cleaner(Command) :
    
    user_options=[]
    
    def __init__(self,dist,**kwargs):
        super(Cleaner,self).__init__(dist,**kwargs)
        
    def initialize_options(self):
        self.directories=[]
        self.files=[]
    
    def finalize_options(self):
        self.directories=['build','dist','wav2mp3.egg-info']
        self.files=[]
    
    def run(self):
        for directory in self.directories:
            try:
                shutil.rmtree(directory)
                print("{} deleted".format(directory))
            except FileNotFoundError:
                print("{} does not exist, so not deleted...".format(directory))
            except Exception as e:
                print("{} : {}".format(e.__class__.__name__,e))
        for file in self.files:
            try:
                os.remove(file)
                print("{} deleted".format(directory))
            except FileNotFoundError:
                print("{} does not exist, so not deleted...".format(directory))
            except Exception as e:
                print("{} : {}".format(e.__class__.__name__,e))
            
        
        


'''
Created on 4 Dec 2017

@author: julianporter
'''

from tempfile import mkdtemp
import os
import shutil
from distutils import ccompiler, sysconfig, errors 
from sys import argv

HelloWorld='''
#include<iostream>
int main(int argc, char* argv[])
{
    std::cout << "hello world" << std::endl;
    return 0;
}
'''

class LibraryError(Exception):
    
    def __init__(self,message,inner=None):
        super(LibraryError,self).__init__()
        self.message=message
        self.inner=inner
        
    def __str__(self):
        if not self.inner:
            st="Library exception: {}"
            args=[self.message]
        else:
            st="Library exception: {}  Inner {} : {}"
            args=[self.message,type(self.inner).__name__,str(self.inner)]
        return st.format(*args)


class CheckLibrary(object):
    
    def __init__(self,*libs):
        self.libraries=libs
        self.tmp=mkdtemp(prefix='_libsearch')
        self.src=os.path.join(self.tmp,'hello.cpp')
        self.exe=os.path.join(self.tmp,'hello')
        self.results=dict()

    def _makeSource(self):
        with open(self.src,'w') as file:
            file.write(HelloWorld)
    
    def _cleanup(self):
        try:
            shutil.rmtree(self.tmp)
        except:
            pass
        
    def test(self):
        try:
            self._makeSource()
        except Exception as e:
            raise LibraryError('Cannot create source file',inner=e)    
        
        try:
            compiler = ccompiler.new_compiler()
            if not isinstance(compiler, ccompiler.CCompiler):
                raise errors.CompileError("Compiler is not valid!")
            sysconfig.customize_compiler(compiler)
            o=compiler.compile([self.src])
            
            for lib in self.libraries:
                try:
                    exe=self.exe+"_"+lib
                    compiler.link_executable(o,exe,libraries=[lib,'stdc++'])
                    self.results[lib]=True
                except Exception as e:
                    self.results[lib]=False
                  
        except errors.CompileError as e:
            raise LibraryError('Compilation problems',inner=e)
        except Exception as e:
            raise LibraryError('General error',inner=e)
        finally:
            self._cleanup()
            
    def __getitem__(self,key):
        return self.results[key]
    
    def __iter__(self):
        return iter(self.results)
    
    
            
    
if __name__=='__main__':
    c=CheckLibrary(*argv[1:])
    c.test()
    for lib in c:
        print("{} : {}".format(lib,c[lib]))

    
    
    
            
        




'''
Created on 5 Dec 2017

@author: julianporter
'''

from wav2mp3 import transcode
from time import perf_counter
from sys import argv


class TestResult(object):
    def __init__(self,rate=0,quality=0,nTests=0,time=0,length=0):
        self.rate=rate
        self.quality=quality
        self.nTests=nTests
        self.time=time
        self.length=length

    def __str__(self):
        return "{}, {}, {}, {}, {}, {}".format(self.rate,self.quality,self.nTests,self.time,self.mean,self.length)
    
    @property
    def mean(self):
        return self.time/self.nTests


class TimeTest(object):
    
    def __init__(self,infile,nTests=100,rate=[8,24,32,48,64,96,128,192],quality=[2,5]):
        with open(infile,"rb") as f:
            self.infile=f.read()
    
        self.nTests=nTests
        self.rate=rate
        self.quality=quality
        self.results=[]
        
    def _test(self,rate,quality):
        count=0
        total=0
        nbytes=0
        for _ in range(self.nTests):
            try:
                start=perf_counter()
                o=transcode(self.infile,bitrate=rate,quality=quality)
                duration=perf_counter()-start
                total+=duration
                nbytes+=len(o)
                count+=1
            except Exception as e:
                print(e)
        return TestResult(rate=rate,quality=quality,nTests=count,time=total,length=nbytes/count)
        
               
    def run(self):
        for rate in self.rate:
            for quality in self.quality:
                print("Testing rate={}, quality={}".format(rate,quality))
                result=self._test(rate,quality)
                self.results.append(result)
                
    def __iter__(self):    
        return iter(self.results)
    
if __name__=='__main__':
    n=int(argv[1])
    t=TimeTest('test.wav',n)
    t.run()
    lines=[str(r) for r in t]
    lines.insert(0, 'rate,quality,nTests,time,mean,length')
    for l in lines: print(l)
    if len(argv)>2:
        lines=[l+'\n' for l in lines]
        with open(argv[2],'w') as f:
            f.writelines(lines)
        




#!/usr/bin/python3
'''
Created on 1 Dec 2017

@author: julianporter
'''
from setuptools import setup, Extension
from setuptools.config import read_configuration
import utils
from collections import namedtuple
from sys import exit


checker=utils.CheckLibrary("mp3lame")
checker.test()
if not checker['mp3lame']:
    print("Cannot build pcm2mp3 unless libmp3lame is installed and on the compiler path")
    exit(1)

configuration=read_configuration('setup.cfg')
metadata=configuration['metadata']


libsrc=['MP3Encoder.cpp','PCMData.cpp','PCMFile.cpp', 'WAVFile.cpp','AIFFFile.cpp', 'Iterator32.cpp',
        'DataChunk.cpp', 'Form.cpp', 'base.cpp', 'Conversions.cpp', 'transcoder.cpp']
wsrc=['lib/'+s for s in libsrc]
wsrc.append('Lame.cpp')

Version = namedtuple('Version',['major','minor','maintenance'])
def processVersion():
    version=metadata['version']
    parts=version.split('.')
    if len(parts)<3: parts.extend([0,0,0])
    return Version(*(parts[:3]))

def makeExtension(module,src):
    print("Making {} with {}".format(module,src))
    
    v=processVersion()
    return Extension(module,
                    define_macros = [('MAJOR_VERSION', v.major),
                                     ('MINOR_VERSION', v.minor),
                                     ('MAINTENANCE_VERSION', v.maintenance)],
                    sources = ['cpp/'+s for s in src],
                    language = 'c++',
                    include_dirs=['/usr/include'],
                    libraries = ['mp3lame'],
                    library_dirs = ['/usr/lib/x86_64-linux-gnu'])

coder = makeExtension('pcm2mp3',wsrc)

with open('README.rst') as readme:
    longDescription = readme.read()

setup (
    ext_modules = [coder],
    long_description = longDescription 
)
